import pytest
import shutil
from pathlib import Path
from tempfile import TemporaryDirectory


@pytest.fixture
def test_project():
    # Copy the test project to a tmp dir to make every test session independent
    with TemporaryDirectory() as tmpdir:
        source_dir = Path(__file__).resolve().parent / "_utils/test_project"
        tmp_project = Path(tmpdir) / "hatch-frozen-test-project"
        shutil.copytree(source_dir, tmp_project)

        pyproject_toml = tmp_project / "pyproject.toml"
        plugin_root_dir = Path(__file__).resolve().parent.parent
        with open(pyproject_toml, "r") as f:
            content = f.read()
            new_content = content.replace(
                "<<<hatch_frozen_cwd>>>", plugin_root_dir.as_uri()
            )
        with open(pyproject_toml, "w") as f:
            f.write(new_content)

        yield tmp_project



import logging
from pathlib import Path
from typing import Any
from hatchling.builders.hooks.plugin.interface import BuildHookInterface


log = logging.getLogger(__name__)


class FrozenDependenciesHook(BuildHookInterface):
    """
    A hook to add pinned dependencies to the build.
    """

    PLUGIN_NAME = "frozen"

    def initialize(self, version: str, build_data: dict[str, Any]):
        """
        Called before the build process starts.
        In this case, build_data is modified to include dependencies
        from requirements.txt.
        """
        requirements_file = "requirements.txt"
        if not Path(requirements_file).exists():
            log.warning(f"{requirements_file} not found. No dependencies injected.")
            return

        dependencies = _parse_requirements_file(requirements_file)
        build_data["dependencies"] = dependencies


def _parse_requirements_file(requirements_file: str):
    with open(requirements_file, "r") as f:
        dependencies = [
            line.strip() for line in f.readlines() if not line.strip().startswith("#")
        ]

        return dependencies



import email
from tempfile import TemporaryDirectory
from pathlib import Path
import zipfile
from _utils import run_hatch_build


def test_frozen_wheel(test_project):
    """
    Test that the plugin freezes the wheel by including
    the dependencies from pyproject.toml
    and the pinned dependencies from requirements.txt.
    (from tests/_utils/test_project)
    """

    run_hatch_build(cwd=test_project)

    # Extract depedencies from the wheel metadata
    wheel_path = test_project / "dist/test_app-1.0.0-py3-none-any.whl"
    metadata = _read_wheel_metadata(wheel_path)
    requires_dist = metadata.get_all("Requires-Dist") or []

    # Verify that the wheel contains:
    # - the dependencies from pyproject
    assert "requests<2.33,>=2.32" in requires_dist
    # - the pinned dependencies from requirements.txt
    assert "urllib3==2.2.2" in requires_dist


def _read_wheel_metadata(wheel_path: Path):
    with TemporaryDirectory() as temp_dir:
        with zipfile.ZipFile(wheel_path, "r") as zip_in:
            zip_in.extractall(temp_dir)

        metadata_path = next(Path(temp_dir).rglob("METADATA"))

        # The METADATA file of the wheel can be parsed with the "email" module
        with open(metadata_path, "r", encoding="utf-8") as f:
            metadata = email.message_from_file(f)
            return metadata



import sys
import subprocess
from os import PathLike
from typing import Union


def run_hatch_build(cwd: Union[str, PathLike[str]]):
    process = subprocess.run([sys.executable, "-m", "hatch", "build"], cwd=cwd)
    process.check_returncode()